@@ -4,7 +4,7 @@ from operator import getitem
 import numpy as np
 
 from .core import getter, getter_nofancy, getter_inline
-from ..blockwise import optimize_blockwise
+from ..blockwise import optimize_blockwise, fuse_roots
 from ..core import flatten, reverse_dict
 from ..optimization import cull, fuse, inline_functions
 from ..utils import ensure_dict
@@ -29,7 +29,7 @@ def optimize(
     rename_fused_keys=True,
     **kwargs
 ):
-    """ Optimize dask for array computation
+    """Optimize dask for array computation
 
     1.  Cull tasks not necessary to evaluate keys
     2.  Remove full slicing, e.g. x[:]
@@ -40,6 +40,7 @@ def optimize(
     # High level stage optimization
     if isinstance(dsk, HighLevelGraph):
         dsk = optimize_blockwise(dsk, keys=keys)
+        dsk = fuse_roots(dsk, keys=keys)
 
     # Low level task optimizations
     dsk = ensure_dict(dsk)
@@ -70,7 +71,7 @@ def optimize(
 
 
 def hold_keys(dsk, dependencies):
-    """ Find keys to avoid fusion
+    """Find keys to avoid fusion
 
     We don't want to fuse data present in the graph because it is easier to
     serialize as a raw value.
@@ -108,7 +109,7 @@ def hold_keys(dsk, dependencies):
 
 
 def optimize_slices(dsk):
-    """ Optimize slices
+    """Optimize slices
 
     1.  Fuse repeated slices, like x[5:][2:6] -> x[7:11]
     2.  Remove full slices, like         x[:] -> x
@@ -195,7 +196,7 @@ def optimize_slices(dsk):
 
 
 def normalize_slice(s):
-    """ Replace Nones in slices with integers
+    """Replace Nones in slices with integers
 
     >>> normalize_slice(slice(None, None, None))
     slice(0, None, 1)
@@ -227,7 +228,7 @@ def check_for_nonfusible_fancy_indexing(fancy, normal):
 
 
 def fuse_slice(a, b):
-    """ Fuse stacked slices together
+    """Fuse stacked slices together
 
     Fuse a pair of repeated slices into a single slice:
 
