@@ -58,7 +58,13 @@ class _iLocIndexer(_IndexerBase):
         if iindexer != slice(None):
             raise NotImplementedError(msg)
 
-        return self._iloc(iindexer, cindexer)
+        if not self.obj.columns.is_unique:
+            # if there are any duplicate column names, do an iloc
+            return self._iloc(iindexer, cindexer)
+        else:
+            # otherwise dispatch to dask.dataframe.core.DataFrame.__getitem__
+            col_names = self.obj.columns[cindexer]
+            return self.obj.__getitem__(col_names)
 
     def _iloc(self, iindexer, cindexer):
         assert iindexer == slice(None)
@@ -277,7 +283,7 @@ class _LocIndexer(_IndexerBase):
 
 
 def _partition_of_index_value(divisions, val):
-    """ In which partition does this value lie?
+    """In which partition does this value lie?
 
     >>> _partition_of_index_value([0, 5, 10], 3)
     0
@@ -297,7 +303,7 @@ def _partition_of_index_value(divisions, val):
 
 
 def _partitions_of_index_values(divisions, values):
-    """ Return defaultdict of division and values pairs
+    """Return defaultdict of division and values pairs
     Each key corresponds to the division which values are index values belong
     to the division.
 
@@ -320,7 +326,7 @@ def _partitions_of_index_values(divisions, values):
 
 
 def _coerce_loc_index(divisions, o):
-    """ Transform values to be comparable against divisions
+    """Transform values to be comparable against divisions
 
     This is particularly valuable to use with pandas datetimes
     """
