@@ -1,13 +1,12 @@
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 import pandas as pd
 
 from .core import Series, DataFrame, map_partitions, apply_concat_apply
 from . import methods
-from .utils import is_categorical_dtype, is_scalar, has_known_categories, PANDAS_VERSION
+from .utils import is_categorical_dtype, is_scalar, has_known_categories
 from ..utils import M
 import sys
+from pandas.api.types import is_list_like
 
 ###############################################################
 # Dummies
@@ -105,17 +104,6 @@ def get_dummies(
     --------
     pandas.get_dummies
     """
-    if PANDAS_VERSION >= "0.23.0":
-        # dtype added to pandas
-        kwargs["dtype"] = dtype
-    elif dtype != np.uint8:
-        # User specified something other than the default.
-        raise ValueError(
-            "Your version of pandas is '{}'. "
-            "The 'dtype' keyword was added in pandas "
-            "0.23.0.".format(PANDAS_VERSION)
-        )
-
     if isinstance(data, (pd.Series, pd.DataFrame)):
         return pd.get_dummies(
             data,
@@ -125,6 +113,7 @@ def get_dummies(
             columns=columns,
             sparse=sparse,
             drop_first=drop_first,
+            dtype=dtype,
             **kwargs
         )
 
@@ -170,6 +159,7 @@ def get_dummies(
         columns=columns,
         sparse=sparse,
         drop_first=drop_first,
+        dtype=dtype,
         **kwargs
     )
 
@@ -183,6 +173,7 @@ def get_dummies(
         sparse=sparse,
         drop_first=drop_first,
         meta=meta,
+        dtype=dtype,
         **kwargs
     )
 
@@ -196,7 +187,8 @@ def pivot_table(df, index=None, columns=None, values=None, aggfunc="mean"):
     """
     Create a spreadsheet-style pivot table as a DataFrame. Target ``columns``
     must have category dtype to infer result's ``columns``.
-    ``index``, ``columns``, ``values`` and ``aggfunc`` must be all scalar.
+    ``index``, ``columns``, and ``aggfunc`` must be all scalar.
+    ``values`` can be scalar or list-like.
 
     Parameters
     ----------
@@ -205,8 +197,8 @@ def pivot_table(df, index=None, columns=None, values=None, aggfunc="mean"):
         column to be index
     columns : scalar
         column to be columns
-    values : scalar
-        column to aggregate
+    values : scalar or list(scalar)
+        column(s) to aggregate
     aggfunc : {'mean', 'sum', 'count'}, default 'mean'
 
     Returns
@@ -230,14 +222,26 @@ def pivot_table(df, index=None, columns=None, values=None, aggfunc="mean"):
             "`df[columns].cat.as_known()` beforehand to ensure "
             "known categories"
         )
-    if not is_scalar(values) or values is None:
-        raise ValueError("'values' must be the name of an existing column")
+    if not (
+        is_list_like(values)
+        and all([is_scalar(v) for v in values])
+        or is_scalar(values)
+    ):
+        raise ValueError("'values' must refer to an existing column or columns")
     if not is_scalar(aggfunc) or aggfunc not in ("mean", "sum", "count"):
         raise ValueError("aggfunc must be either 'mean', 'sum' or 'count'")
 
     # _emulate can't work for empty data
     # the result must have CategoricalIndex columns
-    new_columns = pd.CategoricalIndex(df[columns].cat.categories, name=columns)
+
+    columns_contents = pd.CategoricalIndex(df[columns].cat.categories, name=columns)
+    if is_scalar(values):
+        new_columns = columns_contents
+    else:
+        new_columns = pd.MultiIndex.from_product(
+            (sorted(values), columns_contents), names=[None, columns]
+        )
+
     meta = pd.DataFrame(
         columns=new_columns, dtype=np.float64, index=pd.Index(df._meta[index])
     )
